//
//  Checkers.m
//  Checkers
//
//  Created by Vincent Tanakas on 5/28/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "Checkers.h"

@implementation Checkers

@synthesize currentPlayer = currentPlayer_;

- (id) init
{
  if ((self = [super init]))
  {
    currentPlayer_ = 0;
    playerOneCheckers_ = [NSMutableSet new];
    playerTwoCheckers_ = [NSMutableSet new];
    squares_ = [[NSMutableArray arrayWithCapacity:32] retain];
    for (int i=0;i<32;++i)
    {
      [squares_ addObject:[NSNull null]];
    }
  }
  return self;
}

- (void) dealloc
{
  [playerOneCheckers_ release];
  [playerTwoCheckers_ release];
  [squares_ release];
  [super dealloc];
}

- (id) checkerInRow:(uint)row column:(uint)column
{
  if (row >= 8 || column >= 4)
  {
    return nil; //  Invalid row or column.
  }
  else {
    id object = [squares_ objectAtIndex:[self indexOfRow:row column:column]];
    return (object == [NSNull null]) ? nil : object;
  }
}

- (BOOL) moveCheckerFromRow:(uint)fromRow column:(uint)fromColumn toRow:(uint)toRow column:(uint)toColumn
{
  //  Get checker to move.
  id checkerToMove = [[[self checkerInRow:fromRow column:fromColumn] retain] autorelease];
  int checkerOwner = [self ownerOfChecker:checkerToMove];
  
  if (currentPlayer_ == 0)
  {
    return NO;
  }
  else if (checkerOwner != currentPlayer_)
  {
    return NO;
  }
  else if (currentPlayer_ == 1 && fromRow + 1 != toRow) //  Player 1 must move forward.
  {
    return NO;
  }
  else if (currentPlayer_ == 2 && toRow + 1 != fromRow) //  Player 2 must move backward.
  {
    return NO;
  }
  else if (nil == checkerToMove)
  {
    return NO;
  }
  else if ([self checkerInRow:toRow column:toColumn])
  {
    return NO;
  }
  else if (NO == [self isRow:fromRow column:fromColumn adjacentToRow:toRow column:toColumn])
  {
    return NO;
  }
  else 
  {
    //  Empty the checker's old location and set it at the new location.
    [squares_ replaceObjectAtIndex:[self indexOfRow:fromRow column:fromColumn] withObject:[NSNull null]];
    [squares_ replaceObjectAtIndex:[self indexOfRow:toRow column:toColumn] withObject:checkerToMove];
    
    if (currentPlayer_ == 1)
    {
      currentPlayer_ = 2;
    }
    else if (currentPlayer_ == 2)
    {
      currentPlayer_ = 1;
    }
    
    return YES;
  }  
}

- (BOOL) jumpCheckerFromRow:(uint)fromRow column:(uint)fromColoumn toRow:(uint)toRow column:(uint)toColumn
{
  //  Determine the piece that lies between the two locations.
  return TRUE;
}

- (uint) playerAtRow:(uint)row column:(uint)column
{
  id checker = [self checkerInRow:row column:column];
  if (nil == checker)
  {
    return 0;
  }
  else if ([playerOneCheckers_ containsObject:checker])
  {
    return 1;
  }
  else if ([playerTwoCheckers_ containsObject:checker])
  {
    return 2;
  }
  else {
    //  All checkers in the board should belong to player one or player two.
    NSString *reason = [NSString stringWithFormat:@"The checker at row:%d, column:%d belongs to neither player.", row, column];
    NSException *exception = [NSException exceptionWithName: @"Unclaimed Checker"
                                                     reason: reason
                                                   userInfo: nil];
    @throw exception;
  }
}

- (void) newGame
{
  id newChecker = nil;
  [playerOneCheckers_ removeAllObjects];
  [playerTwoCheckers_ removeAllObjects];
  [squares_ removeAllObjects];
  
  for (int i=0;i<12;++i)
  {
    newChecker = [[NSObject new] autorelease];
    [playerOneCheckers_ addObject:newChecker];
    [squares_ addObject:newChecker];
  }
  for (int i=0;i<8;++i)
  {
    [squares_ addObject:[NSNull null]];
  }
  for (int i=0;i<12;++i)
  {
    newChecker = [[NSObject new] autorelease];
    [playerTwoCheckers_ addObject:newChecker];
    [squares_ addObject:newChecker];
  }
  
  currentPlayer_ = 1;
}

#pragma mark Checkers (PRIVATE)

- (uint) ownerOfChecker:(id)checker
{
  if ([playerOneCheckers_ containsObject:checker])
  {
    return 1;
  }
  else if ([playerTwoCheckers_ containsObject:checker])
  {
    return 2;
  }
  else 
  {
    return 0;
  }
}

- (BOOL) isRow:(uint)fromRow column:(uint)fromColumn adjacentToRow:(uint)toRow column:(uint)toColumn
{
  //  Return true if two tiles are adjacent to each other, regardless of move validity (i.e. forward or backward.)
  if (fromRow == toRow)
  {
    return NO;
  }
  else if (fromRow > toRow && fromRow - toRow > 1)
  {
    return NO;
  }
  else if (toRow > fromRow && toRow - fromRow > 1)
  {
    return NO;
  }
  else if (fromColumn == toColumn)
  {
    return YES;
  }
  else if (fromRow % 2)  //  Odd Row, adjacent tiles have the same column or column + 1.
  {
    if (fromColumn + 1 == toColumn)
    {
      return YES;
    }
    else
    {
      return NO;
    }
  }
  else              //  Even Row, adjacent tiels have the same column or column - 1.
  {
    if (fromColumn == toColumn + 1)
    {
      return YES;
    }
    else
    {
      return NO;
    }
  }
}

- (uint) indexOfRow:(uint)r column:(uint)c
{
  return r * 4 + c;
}

@end
