//
//  CheckersTestsJumping.m
//  Checkers
//
//  Created by Vincent Tanakas on 5/30/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CheckersTestsJumping.h"
#import "Checkers.h"

@implementation CheckersTestsJumping

- (void) prepForJump
{
  //  New Game, and initial moves.
  [checkers newGame];
  [checkers moveCheckerFromRow:2 column:3 toRow:3 column:3];
  [checkers moveCheckerFromRow:5 column:2 toRow:4 column:3];
  
  //  On the next move, player 1 must take a jump from 3,3 to 5,2.
}

- (void) testCantJumpBeforeGameStart
{
  STAssertFalse([checkers jumpCheckerFromRow:2 column:3 toRow:3 column:3],
                @"Player can't jump before game start.");
}

- (void) testCantJumpFromEmptySquare
{
  [checkers newGame];
  [checkers moveCheckerFromRow:2 column:1 toRow:3 column:1];
  STAssertFalse([checkers jumpCheckerFromRow:2 column:1 toRow:4 column:0], @"Can't Jump from Empty Square.");
}

- (void) testCantJumpOverEmptySquare
{
  [checkers newGame];
  STAssertFalse([checkers jumpCheckerFromRow:2 column:0 toRow:4 column:1],
                @"Can't Jump from Over Empty Square.");
}

- (void) testPlayerCanJumpOpponent
{
  [self prepForJump];
  //  Make the jump.
  STAssertTrue([checkers jumpCheckerFromRow:3 column:3 toRow:5 column:2], @"Player 1 can jump player 2's piece.");
}

- (void) testPlayerMustJumpOpponent
{
  //  New Game, and initial moves.
  [self prepForJump];
  
  //  Make a non jumping move.
  STAssertFalse([checkers moveCheckerFromRow:2 column:0 toRow:3 column:0], @"Player 1 must jump player two.");
}

- (void) testCantJumpBackwards
{
  //  Get the game to a state where a backwards jump could occur.
  int mc[] = {
    2,2,3,1,1, 
    5,1,4,1,1,
    2,1,3,0,1,
    4,1,2,2,2,
    1,2,3,1,2,
    5,3,4,3,1,
    3,0,4,1,1,
    6,1,5,1,1,
    2,0,3,0,1,
    5,2,4,2,1,
    3,1,5,2,2,
    5,0,3,1,2};
  int numMoves = sizeof(mc)/sizeof(int);
  
  [checkers newGame];
  for (int i =0;i<numMoves;i+=5)
  {
    if (mc[i+4] == 1)
    {
      [checkers moveCheckerFromRow:mc[i] column:mc[i+1] toRow:mc[i+2] column:mc[i+3]];
    }
    else {
      [checkers jumpCheckerFromRow:mc[i] column:mc[i+1] toRow:mc[i+2] column:mc[i+3]];
    }
  }
  
  //  Last move.
  STAssertFalse([checkers jumpCheckerFromRow:5 column:2 toRow:3 column:3],@"Can't jump backwards.");
}

@end
