//
//  CheckersMovementTests.m
//  Checkers
//
//  Created by Vincent Tanakas on 5/29/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CheckersTestsMovement.h"
#import "Checkers.h"

@implementation CheckersTestsMovement

- (void) setUp
{
  [super setUp];
  [checkers newGame];
}

- (void) testMoveChecker
{
  STAssertTrue([checkers moveCheckerFromRow:2 column:0 toRow:3 column:0], @"Expected move from r2,c0 to r3,c0 to be successful.");
}


- (void) testMoveResultsInCorrectBoardState
{
  STAssertTrue([checkers moveCheckerFromRow:2 column:0 toRow:3 column:0], @"Expected move from r2,c0 to r3,c0 to be successful.");
  STAssertNil([checkers checkerInRow:2 column:0], @"Expected r2,c0 to be empty.");
  STAssertNotNil([checkers checkerInRow:3 column:0], @"Expected r3,c0 to be occupied.");
}

- (void) testMovedCheckerIsSameChecker
{
  id checkerToMove = [checkers checkerInRow:2 column:0];
  [checkers moveCheckerFromRow:2 column:0 toRow:3 column:0];
  id movedChecker = [checkers checkerInRow:3 column:0];
  STAssertEqualObjects(checkerToMove, movedChecker, @"Expected the checker to be the same after move.");
}

- (void) testCantMoveCheckerIntoOccupiedSquare
{
  STAssertFalse([checkers moveCheckerFromRow:0 column:0 toRow:1 column:0], @"Shouldn't be able to move checker into occupied square.");
}

- (void) testCantMoveSecondCheckerIntoRecentlyOccupiedSquare
{
  STAssertTrue([checkers moveCheckerFromRow:2 column:1 toRow:3 column:0], @"Expected move to be succeed.");
  STAssertFalse([checkers moveCheckerFromRow:2 column:0 toRow:3 column:0], @"Expected move into occupied square to fail.");
}

- (void) testCantMoveCheckerToSameLocation
{
  STAssertFalse([checkers moveCheckerFromRow:2 column:0 toRow:2 column:0], @"Shouldn't be able to move checker to its current location.");
}

- (void) testCantMoveTwoRowsForward
{
  STAssertFalse([checkers moveCheckerFromRow:2 column:0 toRow:4 column:0], @"Shouldn't be able to move checker two rows forward.");
}

- (void) testCantMoveZeroRowsForward
{
  [checkers moveCheckerFromRow:2 column:0 toRow:3 column:0];
  STAssertFalse([checkers moveCheckerFromRow:3 column:0 toRow:3 column:1], @"Shouldn't be able to move checker zero rows forward.");
}

- (void) testCantMoveOneRowBackward
{
  [checkers moveCheckerFromRow:2 column:0 toRow:3 column:0];
  STAssertFalse([checkers moveCheckerFromRow:3 column:0 toRow:2 column:0], @"Shouldn't be able to move checker back one row.");
}

- (void) testCantMoveCheckerToNonAdjacentSquare
{
  STAssertFalse([checkers moveCheckerFromRow:2 column:0 toRow:3 column:1], @"Shouldn't be able to move checker in 2,0 to 3,1, since it's not adjacent.");
}

@end
