//
//  CheckersTestsPlayers.m
//  Checkers
//
//  Created by Vincent Tanakas on 5/29/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CheckersTestsPlayers.h"
#import "Checkers.h"

@implementation CheckersTestsPlayers

- (void) testCheckerInLowerLeftCornerIsPlayerOne
{
  uint expectedIndex = 1;
  [checkers newGame];
  uint playerIndex = [checkers playerAtRow:0 column:0];
  STAssertEquals(playerIndex, expectedIndex, @"Expected checker at 0,0 to belong to player one.");
}

- (void) testCheckerInUpperRightCornerIsPlayerTwo
{
  uint expectedIndex = 2;
  [checkers newGame];
  uint playerIndex = [checkers playerAtRow:7 column:3];
  STAssertEquals(playerIndex, expectedIndex, @"Expected checker at 7,3 to belong to player two.");
}

- (void) testEmptySquareBelongsToNobody
{
  uint expectedIndex = 0;
  uint playerIndex = [checkers playerAtRow:3 column:0];
  STAssertEquals(playerIndex, expectedIndex, @"Expected checker at 3,0, to belong to nobody.");
}

- (void) testNobodysTurnBeforeGameStart
{
  uint expectedIndex = 0;
  uint playerIndex = [checkers currentPlayer];
  STAssertEquals(playerIndex, expectedIndex, @"Expected it to be nobody's turn before game starts.");
}

- (void) testAfterGameStartPlayerOnesTurn
{
  [checkers newGame];
  uint expectedIndex = 1;
  uint playerIndex = [checkers currentPlayer];
  STAssertEquals(playerIndex, expectedIndex, @"Expected it to be nobody's turn before game starts.");
}

- (void) testPlayerTwosTurnAfterPlayerOneMoves
{
  [checkers newGame];
  STAssertTrue([checkers moveCheckerFromRow:2 column:0 toRow:3 column:0],@"Player One is able to move first...");
  
  uint expectedIndex = 2;
  uint playerIndex = [checkers currentPlayer];
  STAssertEquals(playerIndex, expectedIndex, @"Expected it to be player 2's turn after player one moves.");
}

- (void) testP1CantMoveP2Checker
{
  [checkers newGame];
  STAssertFalse([checkers moveCheckerFromRow:5 column:0 toRow:4 column:0], @"Player 1 can't move Player 2's checker.");
}

- (void) testP2CantMoveP1Checker
{
  [checkers newGame];
  [checkers moveCheckerFromRow:2 column:0 toRow:3 column:0];
  STAssertFalse([checkers moveCheckerFromRow:2 column:1 toRow:3 column:1], @"Player 2 can't move Player 1's checker.");
}

- (void) testPlayersALternateMoves
{
  [checkers newGame];
  STAssertTrue([checkers moveCheckerFromRow:2 column:0 toRow:3 column:0], @"Player 1 moves...");
  STAssertTrue([checkers moveCheckerFromRow:5 column:0 toRow:4 column:0], @"Player 2 moves...");
}

- (void) testPlayerCantMoveTwice
{
  [checkers newGame];
  STAssertTrue([checkers moveCheckerFromRow:2 column:0 toRow:3 column:0], @"Player 1 moves...");
  STAssertFalse([checkers moveCheckerFromRow:2 column:1 toRow:3 column:1], @"Player 1 can't move again...");
}



@end
