//
//  CheckersTests.m
//  CheckersTests
//
//  Created by Vincent Tanakas on 5/28/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "CheckersTestsPosition.h"
#import "Checkers.h"

@implementation CheckersTestsPosition

static const size_t SIZE_OF_TUPLE = sizeof(uint)*2;

- (void) testBoardIsEmptyBeforeGameStart
{
  for (int row=0;row<8;++row)
  {
    for (int col=0;col<4;++col)
    {
      STAssertNil([checkers checkerInRow:row column:col], @"Expected checker in %d,%d to be nil before game start.", row, col);
    }
  }
}

- (void) testIsCheckerInRow0Column0
{
  [checkers newGame];
  id checker = [checkers checkerInRow:0 column:0];
  STAssertNotNil(checker, @"Expected non-nil checker in the lower left corner.");
}

- (void) testIsCheckerInRow7Column3
{
  [checkers newGame];
  id checker = [checkers checkerInRow:7 column:3];
  STAssertNotNil(checker, @"Expected non-nil checker in upper-right corner.");
}

- (void) testNoCheckerInRow4Column0
{
  [checkers newGame];
  id checker = [checkers checkerInRow:3 column:0];
  STAssertNil(checker, @"Expected nil checker in row 3, column 0.");
}

- (void) testNoCheckerInRow8Column0
{
  [checkers newGame];
  id checker = [checkers checkerInRow:8 column:0];
  STAssertNil(checker, @"Expected nil checker in row 8, column 0.");
}

- (void) testNoCheckerInRow0Column4
{
  [checkers newGame];
  id checker = [checkers checkerInRow:0 column:4];
  STAssertNil(checker, @"Expected nil checker in row 0, column 4.");
}

- (void) testCheckerInRow0Column0_sameAs_checkerInRow0Column0
{
  [checkers newGame];
  id checker1 = [checkers checkerInRow:0 column:0];
  id checker2 = [checkers checkerInRow:0 column:0];
  STAssertEqualObjects(checker1, checker2, @"Expected checkers to be the same object.");
}

- (void) testCheckerInRow0Column0_differentFrom_checkerInRow1Column0
{
  [checkers newGame];
  id checker1 = [checkers checkerInRow:0 column:0];
  id checker2 = [checkers checkerInRow:1 column:0];
  STAssertFalse(checker1 == checker2, @"Expected checkers to be different objects.");
}

- (void) testNearPiecesInInitialPositions
{
  //  32 squares on a checkers board, 24 of which are occupied.
  //  Tupples are arranged row, column, row, column, ...
  uint nearPlayerIndexes[] = {0,0, 0,1, 0,2, 0,3, 1,0, 1,1, 1,2, 1,3, 2,0, 2,1, 2,2, 2,3}; 
  size_t numberOfNearTuples = sizeof(nearPlayerIndexes)/SIZE_OF_TUPLE; //  Two uints per tuples.
  STAssertEquals(numberOfNearTuples, (size_t)12, @"Expected their to be 24 tuples for the near player.");
  
  [checkers newGame];
  for (size_t i=0;i<numberOfNearTuples;++i)
  {
    uint row = nearPlayerIndexes[i*2];
    uint column = nearPlayerIndexes[i*2+1];
    id checker = [checkers checkerInRow:row column:column];
    STAssertNotNil(checker, @"Expected non-nil checker at row:%d, column:%d", row, column);
  }
}

- (void) testFarPiecesInInitialPositions
{
  uint farPlayerIndexes[] = {5,0, 5,1, 5,2, 5,3, 6,0, 6,1, 6,2, 6,3, 7,0, 7,1, 7,2, 7,3};
  size_t numberOfFarTuples = sizeof(farPlayerIndexes)/SIZE_OF_TUPLE; //  Two uints per tuples.
  STAssertEquals(numberOfFarTuples, (size_t)12, @"Expected their to be 24 tuples for the far player.");  
  
  
  [checkers newGame];
  for (size_t i=0;i<numberOfFarTuples;++i)
  {
    uint row = farPlayerIndexes[i*2];
    uint column = farPlayerIndexes[i*2+1];
    id checker = [checkers checkerInRow:row column:column];
    STAssertNotNil(checker, @"Expected non-nil checker at row:%d, column:%d", row, column);
  }
}

- (void) testEmptyInitialPositions
{
  uint emptyIndexes[] = {3,0, 3,1, 3,2, 3,3, 4,0, 4,1, 4,2, 4,3};
  size_t numberOfEmptyTuples = sizeof(emptyIndexes)/SIZE_OF_TUPLE; //  Two uints per tuples.
  STAssertEquals(numberOfEmptyTuples, (size_t)8, @"Expected their to be 16 tuples for empty spaces.");
  
  
  [checkers newGame];
  for (size_t i=0;i<numberOfEmptyTuples;++i)
  {
    uint row = emptyIndexes[i*2];
    uint column = emptyIndexes[i*2+1];
    id checker = [checkers checkerInRow:row column:column];
    STAssertNil(checker, @"Expected nil checker at row:%d, column:%d", row, column);
  }
}





@end
